/**
 * apeman task to render jade files.
 * @memberof module:apeman-task-jade/lib
 * @function define
 * @param {string|string[]} srcDir - Source directory path.
 * @param {options} [options] - Optional settings.
 * @param {boolean} [options.hierarchy=true] - keep directory hierarchy when a directory is specified.
 * @param {string} [options.out] - Output directory path.
 * @param {string|object} [options.locals] - Locals data file.
 * @param {string} [options.pretty] - Pretty html.
 * @returns {function} - Defined task function.
 */

"use strict";

var argx = require('argx'),
    path = require('path'),
    mkdirp = require('mkdirp'),
    async = require('async'),
    fs = require('fs'),
    execcli = require('execcli'),
    objnest = require('objnest');

/** @lends define */
function define(src, options) {
    var args = argx(arguments);
    src = [].concat(args.shift('string|array') || []);
    options = objnest.expand(args.pop('object') || {});

    var out = options.out || process.cwd(),
        pretty = !!options.pretty,
        hierarchy = options.hierarchy !== false,
        locals = options.locals || {};

    var tmp = path.resolve(out, '.jade-locale.' + new Date().getTime() + '.tmp.json');

    /**
     * Defined task.
     * @function task
     * @param {object} context - Apeman task context.
     * @param {function} callback - Callback when done.
     */
    function task(context, callback) {
        var logger = context.logger,
            verbose = context.verbose;
        async.series([
            function(callback){
                mkdirp(out, callback);
            },
            function (callback) {
                if (typeof(locals) === 'string') {
                    locals = require(path.resolve(locals));
                }
                fs.writeFile(tmp, JSON.stringify(locals), callback);
            },
            function (callback) {
                var args = [
                    {
                        out: out,
                        pretty: pretty,
                        obj: tmp,
                        hierarchy: hierarchy
                    }].concat(src);
                execcli('jade', args, {
                    notfound: 'try `npm install jade -g`'
                }, callback);
            },
            function (callback) {
                fs.unlink(tmp, callback);
            }
        ], callback);

    }

    task.$desc = "Compile jade files.";
    return task;
}

module.exports = define;
